/*
 * XeviGetVisualInfo.c  
 *
 * Copyright (c) 2005 Intel Corporation
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * Intel Corporation DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 * 

 */
/*
 * SYNOPSIS:
 *   int
 *   XeviGetVisualInfo()
 */

/* $XConsortium: mcinclude.mc,v 1.3 92/07/07 17:07:26 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/11 19:34:01 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XeviGetVisualInfo	1
char    *TestName = "XeviGetVisualInfo";



static int 	ValueReturn;

#include <X11/Xlibint.h>
#include <X11/extensions/XEVI.h>
#include <X11/extensions/XEVIstr.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>
#include <X11/Xutil.h>
static XExtensionInfo *xevi_info;/* needs to move to globals.c */
static /* const */ char *xevi_extension_name = EVINAME;
#define XeviCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, xevi_extension_name, val)
/*****************************************************************************
 *                                                                           *
 *                         private utility routines                          *
 *                                                                           *
 *****************************************************************************/
static /* const */ XExtensionHooks xevi_extension_hooks = {
    NULL,                       /* create_gc */
    NULL,                       /* copy_gc */
    NULL,                       /* flush_gc */
    NULL,                       /* free_gc */
    NULL,                       /* create_font */
    NULL,                       /* free_font */
    NULL,                       /* close_display */
    NULL,                       /* wire_to_event */
    NULL,                       /* event_to_wire */
    NULL,                       /* error */
    NULL,                       /* error_string */
};
static XEXT_GENERATE_FIND_DISPLAY (find_display, xevi_info,
                                   xevi_extension_name,
                                   &xevi_extension_hooks, 0, NULL)
static Display *client2 = NULL;
static ExtendedVisualInfo	*evi_return = NULL;
static XExtDisplayInfo *info = NULL;
static int count = -1;


int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XeviGetVisualInfo-1.(A)
 * With the exception of XeviQueryExtension, if any of these
 * routines are called with a display that does not support the
 * extension, the ExtensionErrorHandler (which can be set with
 * XSetExtensionErrorHandler and functions the same way as
 * XSetErrorHandler) will be called and the function will then
 * return. n; Open the display If the display supports the EVI
 * extension - munge the structure so it appears the extension
 * is not supported End Call the XeviGetVisualInfo function and
 * verify error handling Close the Display
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();


	client2 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client2 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( Success != XeviGetVisualInfo( client2, NULL, 0, &evi_return, &count ) )
	{
		delete("XeviGetVisualInfo should not have failed. Test can not proceed" );
		return;
	}
	else
		CHECK;
	/* XCloseDisplay( client2 ); */
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XeviGetVisualInfo-2.(A)
 * XeviGetVisualInfo returns a list of ExtendedVisualInfo
 * structures that describe visual information beyond that
 * supported by the core protocol
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();


        if( (Display *)NULL ==  client2 ) 
	{
		client2 = XOpenDisplay( config.display );
		if( (Display *)NULL ==  client2 ) 
		{
			delete("could not create new client");
			return;
		}
	}
	CHECK;

	if ( (VisualID *)NULL == evi_return )
	{
		if ( Success != XeviGetVisualInfo( client2, NULL, 0, &evi_return, &count ) )
		{
			report("XeviGetVisualInfo should not have failed." );
			XCloseDisplay( client2 );
			FAIL; 
		}
	}	
	CHECK;
	/* XCloseDisplay( client2 ); */
	CHECKPASS(2);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XeviGetVisualInfo-3.(A)
 * XeviGetVisualInfo returns Success if successful, or an X
 * error otherwise
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();

	int i;

        if( (Display *)NULL ==  client2 ) 
	{
		client2 = XOpenDisplay( config.display );
		if( (Display *)NULL ==  client2 ) 
		{
			delete("could not create new client");
			return;
		}
	}

	if ( (VisualID *)NULL == evi_return )
	{
		if ( XeviGetVisualInfo( client2, NULL, 0, &evi_return, &count ) != Success )
		{
			report("XeviGetVisualInfo should not have failed." );
			FAIL; 
		}
	}
	CHECK;

	/*
         * we need to validate return informations
	 */

	report( "XeviGetVisualInfo returned %d structures", count );
	if ( count < 0 )
	{
		report("Invalid count returned from XeviGetVisualInfo." );
		FAIL;
	} else
		CHECK;
/* would be good to verify the evi_return structures
	for ( i = 0; i < count; i++ )
	{
		if ( NULL == evi_return[i] )
		{
			report( "Invalid evi_return from XeviGetVisualInfo." );
			FAIL;
		}
		else
			CHECK;
	}
*/

	/* XCloseDisplay( client2 ); */
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XeviGetVisualInfo-4.(A)
 * If the argument visual is NULL, then information for all
 * visuals of all screens is returned. Otherwise, it's a
 * pointer to a list of visuals for which extended visual
 * information is desired
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();


	Display *client3;
	ExtendedVisualInfo	*evi_return3;
	VisualID	*visual;
	int i;
	int rc;

	client3 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client3 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	report( "xevigetvisualinfo returned %d structures", count );
	visual = malloc( count * sizeof( VisualID ) );
	if ( (VisualID *)NULL == visual )
	{
                delete("could not create new visual ids");
                return;
        }  else 
		CHECK;
	
	for( i = 0; i < count; i++ )
	{	
		visual[i] = evi_return[i].core_visual_id;
	}

	CHECK;

	if ( Success != ( rc =  XeviGetVisualInfo( client3, &visual, (i-1), &evi_return3, &count ) ) )
	{
		/* until we can call XeviGetVisualInfo more than once,
		   this is going to fail */
		report("XeviGetVisualInfo should not have failed." );
		report("return code is %d.", rc );
		unsupported("This assertion can only be tested after Freedesktop.org bug number 3465 has been resolved\n" ); 
		return;
	}
	else
		CHECK;


	XCloseDisplay( client3 );
	CHECKPASS(4);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XeviGetVisualInfo-5.(A)
 * XeviGetVisualInfo will return BadValue if passed an illegal
 * visual ID, BadAccess if the X server does not respond,
 * BadAlloc if there is a memory allocation failure.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();

	/*
         * Not checking for server failuer or malloc failure
	 */

	Display *client4;
	ExtendedVisualInfo	*evi_return4;
	VisualID	*visual;
	int count;
	int rc;

	visual = malloc( 10 * sizeof( VisualID ) );
	client4 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client4 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( (rc = XeviGetVisualInfo( client4, visual, 3, &evi_return4, &count ) ) == Success )
	{
		report("XeviGetVisualInfo should have failed." );
		FAIL; 
	}
	else
		CHECK;

	if ( rc != BadValue )
	{
		report("rc from XeviGetVisualInfo should have been BadValue." );
		FAIL;
	} 
	else
		CHECK;

	XCloseDisplay( client2 );
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
