/*
 * XeviQueryVersion.c  
 *
 * Copyright (c) 2005 Intel Corporation
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * Intel Corporation DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 * 

 */
/*
 * SYNOPSIS:
 *   Bool
 *   XeviQueryVersion()
 */

/* $XConsortium: mcinclude.mc,v 1.3 92/07/07 17:07:26 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/11 19:34:01 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XeviQueryVersion	1
char    *TestName = "XeviQueryVersion";



#include <X11/Xlibint.h>
#include <X11/extensions/XEVI.h>
#include <X11/extensions/XEVIstr.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>
#include <X11/Xutil.h>
static XExtensionInfo *xevi_info;/* needs to move to globals.c */
static /* const */ char *xevi_extension_name = EVINAME;
#define XeviCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, xevi_extension_name, val)
/*****************************************************************************
 *                                                                           *
 *                         private utility routines                          *
 *                                                                           *
 *****************************************************************************/
static /* const */ XExtensionHooks xevi_extension_hooks = {
    NULL,                       /* create_gc */
    NULL,                       /* copy_gc */
    NULL,                       /* flush_gc */
    NULL,                       /* free_gc */
    NULL,                       /* create_font */
    NULL,                       /* free_font */
    NULL,                       /* close_display */
    NULL,                       /* wire_to_event */
    NULL,                       /* event_to_wire */
    NULL,                       /* error */
    NULL,                       /* error_string */
};
static XEXT_GENERATE_FIND_DISPLAY (find_display, xevi_info,
                                   xevi_extension_name,
                                   &xevi_extension_hooks, 0, NULL)


int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XeviQueryVersion-1.(A)
 * XeviQueryVersion returns True if the request succeeded; the
 * values of the major and minor protocol version supported by
 * the server are returned in major_version_return and
 * minor_version_return.
 */
static void
t001()
{
	Display *client2;
	int major = -1;
	int minor = -1;
int 	pass = 0, fail = 0;

	tpstartup();

	client2 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client2 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( !XeviQueryExtension( client2 ) )
	{
		delete( "Display does not support extensions." );
		XCloseDisplay( client2 );
		return;
	} else 
		CHECK;

	if ( !XeviQueryVersion( client2, &major, &minor ) ||
		0 == major  || -1 == major || -1 == minor )
	{
		/* Test has failed */
		report( "Failed to obtain EVI vesion or version numbers are invalid." );
		FAIL;
	} else 
		CHECK;

	XCloseDisplay( client2 );
	CHECKPASS(3);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XeviQueryVersion-2.(A)
 * With the exception of XeviQueryExtension, if any of these
 * routines are called with a display that does not support the
 * extension, the ExtensionErrorHandler (which can be set with
 * XSetExtensionErrorHandler and functions the same way as
 * XSetErrorHandler) will be called and the function will then
 * return.
 */
static void
t002()
{
	Display *client2;
	int major = -1;
	int minor = -1;
	XExtDisplayInfo *info;
int 	pass = 0, fail = 0;

	tpstartup();

	client2 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client2 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( XeviQueryExtension( client2 ) )
	{
		unsupported( "This assertion can only be verified with a display that does not support EVI extensions." );
		return;
	} else 
		CHECK;

	XCloseDisplay( client2 );
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
