/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xlib17/unnrgn/unnrgn.m,v 1.2 2005-11-03 08:43:14 jmichael Exp $
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   
 *   XUnionRegion(sra, srb, dr_return)
 *   Region		sra;
 *   Region		srb;
 *   Region		dr_return;
 */

/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcinclude.mc,v 1.2 2005-02-12 17:50:23 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcextern.mc,v 1.1.1.1 2005-02-12 14:37:14 anderson Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUnionRegion	1
char    *TestName = "XUnionRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XUnionRegion function
 */
static Region		sra;
static Region		srb;
static Region		dr_return;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	sra = 0;
	srb = 0;
	dr_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

int			i;
int			j;
Region			R1;
Region			R2;
Region			R3;
Region			Rt;
static XPoint		tri1[] = { {5,5}, {30,5}, {5,20} };
static XPoint		tri2[] = { {5,20}, {30,20}, {30,5} };
static XRectangle	rect1 = { 5,5, 25,15};
XRectangle		rect2;
XRectangle		rrect;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XUnionRegion-1.(A)");
	report_assertion("A call to XUnionRegion computes the union of the regions");
	report_assertion("sra and srb and places the result in the region dr_return.");

	report_strategy("Create regions R1 R2 R3 using XCreateRegion.");
	report_strategy("Set regions R1 and R2 to right-angled triangles joined along the hypotenuse using XPolygonRegion.");
	report_strategy("Set R3 to the rectangle formed by joining the triangles using XUnionRectWithRegion.");
	report_strategy("Obtain the union of R1 and R2 using XUnionRegion.");
	report_strategy("Verify that union is the same as R3 using XEqualRegion.");
	report_strategy("");
	report_strategy("Create a region using XCreateRegion.");
	report_strategy("Set the region to an overlapping set of 3x3 rectangular regions using XUnionRegion.");
	report_strategy("Verify that the result is the correct rectangular region using XEqualRegion.");

	tpstartup();
	setargs();
	R1 = makeregion();
	R2 = makeregion();
	R3 = makeregion();
	Rt = makeregion();

	if(isdeleted()) return;

	R1 = XPolygonRegion(tri1, NELEM(tri1), WindingRule);
	R2 = XPolygonRegion(tri2, NELEM(tri2), WindingRule);

	sra = R1;
	srb = R2;
	dr_return = R3;
	startcall(Dsp);
	if (isdeleted())
		return;
	XUnionRegion(sra, srb, dr_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUnionRectWithRegion(&rect1, Rt, Rt);

	if( XEqualRegion(Rt, R3) != True) {
		report("%s() did not unify two triangular regions into the correct rectangular region.", TestName);
		FAIL;
	} else
		CHECK;


	R2 = makeregion();

	srb = R2;
	dr_return = R2;
	rect2.width=3;
	rect2.height=3;
	for(j=5; j<18; j++) {
		rect2.y=j;
		for(i=5; i<28; i++) {

			if((R1 = XCreateRegion()) == (Region) NULL) {
				delete("XCreateRegion() returned NULL.");
				return;
			} else
				if(i==27 && j==17) CHECK;

			rect2.x=i;
			XUnionRectWithRegion(&rect2, R1, R1);
			sra = R1;
			startcall(Dsp);
			if (isdeleted())
				return;
			XUnionRegion(sra, srb, dr_return);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			XDestroyRegion(R1);
		}
	}
	
	if( XEqualRegion(Rt, R2) != True) {
		report("%s() did not unify a set of over-lapping rectangular regions into the correct rectangular region.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
