/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xlib7/qryclrs/qryclrs.m,v 1.3 2005-11-03 08:43:44 jmichael Exp $
* 
* Copyright (c) 2002 The Open Group
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   void
 *   XQueryColors(display, colormap, defs_in_out, ncolors)
 *   Display	*display;
 *   Colormap colormap;
 *   XColor	*defs_in_out;
 *   int	ncolors;
 */

/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcinclude.mc,v 1.2 2005-02-12 17:50:23 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcextern.mc,v 1.1.1.1 2005-02-12 14:37:14 anderson Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryColors	1
char    *TestName = "XQueryColors";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XQueryColors function
 */
static Display	*display;
static Colormap colormap;
static XColor	*defs_in_out;
static int	ncolors;


#include <X11/Xproto.h>
XColor	dummycol;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	defs_in_out = &dummycol;
	ncolors = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

XVisualInfo 	*vp;
XColor		*cellptr, *defptr, *refptr, *acptr;
int		size;
unsigned long	i;
unsigned long 	vmask = 0L;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XQueryColors-1.(A)");
	report_assertion("A call to XQueryColors obtains the RGB values of the");
	report_assertion("colourmap entries specified by the pixel components of the");
	report_assertion("XColor structures named by the defs_in_out argument, and");
	report_assertion("returns the RGB values in the red, green, and blue");
	report_assertion("components and sets the flags components to the bitwise OR");
	report_assertion("of DoRed, DoGreen, and DoBlue.");

	report_strategy("For each supported visual class :");
	report_strategy("  Create a colormap with XCreateColormap with alloc = AllocNone.");
	report_strategy("  Allocate a new r/o colourmap cell with XAllocColor.");
	report_strategy("  Obtain the rgb values and flags components with XQueryColors using the pixel returned by XAllocCOlor.");
	report_strategy("  Verify that the returned flags component was set to DORed|DoGreen|DoBlue.");
	report_strategy("  Verify that the rgb values are identical to those returned by XAllocColor.");

	tpstartup();
	setargs();
	if((vmask = visualsupported(display, vmask)) != 0L)
		for(resetsupvis(vmask); nextsupvis(&vp); ) {
			size = vp->colormap_size;
			if (vp->class == DirectColor)
				size = maxsize(vp);
			/*
			 * Ensure we won't exceed the longest possible
			 * QueryColors protocol request.  (Can happen
			 * for StaticGray visuals with depth 16.)
			 */
			if (size > 65535 - sizeof(xQueryColorsReq))
				size = 65535 - sizeof(xQueryColorsReq);

			colormap = makecolmap(display, vp->visual, AllocNone);

			defptr = (XColor *) malloc( size * sizeof(XColor));
			refptr = (XColor *) malloc( size * sizeof(XColor));
			
			if( (defptr == (XColor *) 0) || (refptr == (XColor *) 0)) {
				delete("malloc() failed to allocate space for the colour array.");
				return;
			}

			for(i=0, cellptr = defptr; i<size; i++, cellptr++) {  /* May or may not allocate the entire colourmap */
				cellptr->pixel = ~0L;
				cellptr->red = i<<8;
				cellptr->green = i<<8;
				cellptr->blue =  i<<8;
				cellptr->flags = 0;

				if(XAllocColor(display, colormap, cellptr) == False) {
					delete("XAllocColor() failed.");
					return;
				}

				refptr[i] = defptr[i];
			} 
			if(i == size)
				CHECK;

			ncolors = size;
			defs_in_out = defptr;
			startcall(display);
			if (isdeleted())
				return;
			XQueryColors(display, colormap, defs_in_out, ncolors);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			if(geterr() == Success)
				for(i=0, cellptr = defptr, acptr = refptr; i<size; i++, cellptr++, acptr++) {
					if(cellptr->flags != (DoRed|DoGreen|DoBlue)) {
						report("XQueryColors() did not set the flags of cell %u to (DoRed|DoGreen|DoBlue)", cellptr->pixel);
						FAIL;
					}

					if((cellptr->pixel != acptr->pixel) || (cellptr->red != acptr->red) ||
							(cellptr->green != acptr->green) || (cellptr->blue != acptr->blue)){
						report("XQueryColors() returned pixel %u  r %u g %u b %u instead of pixel %u r %u g %u b %u",
							cellptr->pixel, cellptr->red, cellptr->green, cellptr->blue,
							acptr->pixel, acptr->red, acptr->green, acptr->blue);
						FAIL;
					} 
				}
			else
				FAIL;	

			if(i == size)
				CHECK;

			free(defptr);
			free(refptr);
	}

	CHECKPASS(2*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

static void t002(){

XVisualInfo 	*vp;
XColor		tcol;
XColor		qcol[3];
unsigned long	vmask;
int 	pass = 0, fail = 0;

 	report_purpose(2);

	report_assertion("Assertion XQueryColors-2.(A)");
	report_assertion("When the pixel component in one or more of the ncolors");
	report_assertion("XColor structures named by the defs_in_out argument is not a");
	report_assertion("valid entry in the colormap argument, then a BadValue error");
	report_assertion("occurs which will report any one of the invalid pixel");
	report_assertion("values.");

	report_strategy("For each supported visual class:");
	report_strategy("  Create a colormap with XCreateColormap with alloc set to AllocNone.");
	report_strategy("  Allocate a r/o colormap cell with XAllocColor.");
	report_strategy("  Call XQueryColors with pixel array comprising pixel components 0L, -1L, -2L.");
	report_strategy("  Verify that a BadValue error occurs.");
	report_strategy("  Verify that the reported BadValue was either -1 or -2.");

	tpstartup();
	setargs();
	if((vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocNone);
		XAllocColor(display, colormap, &tcol);
		defs_in_out = qcol;
		qcol[0].pixel = tcol.pixel;
		qcol[1].pixel = (unsigned long)-1L;
		qcol[2].pixel = (unsigned long)-2L;
		ncolors = 3;
		startcall(display);
		if (isdeleted())
			return;
		XQueryColors(display, colormap, defs_in_out, ncolors);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue) {
			if((getbadvalue() != (unsigned long)-1) && (getbadvalue() != (unsigned long)-2)) {
				report("BadValue reported was neither -1 or -2");
				FAIL;
			} else
				CHECK;
		}

	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
