/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xproto/strnmdclr/strnmdclr.m,v 1.3 2005-11-03 08:44:21 jmichael Exp $
* 
* Copyright (c) 2001 The Open Group
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Copyright 1990, 1991 by UniSoft Group Limited.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
* Copyright 1989 by Sequent Computer Systems, Inc., Portland, Oregon
* 
* 			All Rights Reserved
* 
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose and without fee is hereby granted,
* provided that the above copyright notice appears in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of Sequent not be used
* in advertising or publicity pertaining to distribution or use of the
* software without specific, written prior permission.
* 
* SEQUENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
* ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
* SEQUENT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
* ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
* WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
* ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
* SOFTWARE.
* 
*/
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcxpinc.mc,v 1.1.1.1 2005-02-12 14:37:14 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"tet_api.h"
#ifdef INPUTEXTENSION
#include	"X11/extensions/XIproto.h"
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif


#define T_StoreNamedColor	1
char    *TestName = "StoreNamedColor";



/* Touch test for StoreNamedColor request */

#include "Xstlib.h"

#define CLIENT 0
static TestType test_type = SETUP;
xResourceReq *req;
xAllocColorCellsReply *acreply;
xAllocNamedColorReply *anreply;
xError *err;
extern CARD32 reply_pixel;

/* 
   intent:	If a read-write color cell can be allocated:
		Allocate read-write cell by sending an AllocColorCells request,
		send a StoreNamedColor request to the server and check no errors,
   input:	 
   output:	 none
   global input: 
   side effects: creates a window resource
                 writes to global reply_pixel
   methods:	 
*/

static
void
tester_rw()
{
	Create_Client(CLIENT);
	if (!Allocatable (CLIENT)) {
		unsupported("The visual class for screen zero is not writable");
		Exit_OK();
	}

	(void) Create_Default_Window(CLIENT);
	(void) Create_Default_Colormap(CLIENT);

	/* allocate a color and store pixel from reply in global */

	req = (xResourceReq *) Make_Req(CLIENT, X_AllocColorCells);
	Send_Req(CLIENT, (xReq *) req);
	Log_Trace("client %d sent default AllocColorCells request\n", CLIENT);
	if ((acreply = (xAllocColorCellsReply *) Expect_Reply(CLIENT, X_AllocColorCells)) == NULL) {
		Log_Err("client %d failed to receive AllocColorCells reply\n", CLIENT);
		Exit();
	}  else  {
		Log_Trace("client %d received AllocColorCells reply\n", CLIENT);
		/* 
		 * This line gets the first pixel value OR'd with 
		 * zero of the returned masks
		 */
		reply_pixel = *(CARD32 *) (acreply + 1);
		Free_Reply(acreply);
	}

	(void) Expect_Nothing(CLIENT);
	Free_Req(req);

	/* store a color in the pixel from the reply */

	req = (xResourceReq *) Make_Req(CLIENT, X_StoreNamedColor);
	Send_Req(CLIENT, (xReq *) req);
	Log_Trace("client %d sent default StoreNamedColor request\n", CLIENT);
	(void) Expect_Nothing(CLIENT);
	Free_Req(req);
	
	Exit_OK();
}

/* 
   intent:	Allocate read-only cell by sending an AllocNamedColor request,
		send a StoreNamedColor request to the server and check 
                that a BadAccess error is sent back.
   input:	 
   output:	 none
   global input: 
   side effects: creates a window resource
                 writes to global reply_pixel
   methods:	 
*/
static
void
tester()
{
	Create_Client(CLIENT);

	(void) Create_Default_Window(CLIENT);
	(void) Create_Default_Colormap(CLIENT);

	/* allocate a color and store pixel from reply in global */

	req = (xResourceReq *) Make_Req(CLIENT, X_AllocNamedColor);
	Send_Req(CLIENT, (xReq *) req);
	Log_Trace("client %d sent default AllocNamedColor request\n", CLIENT);
	if ((anreply = (xAllocNamedColorReply *) Expect_Reply(CLIENT, X_AllocNamedColor)) == NULL) {
		Log_Err("client %d failed to receive AllocNamedColor reply\n", CLIENT);
		Exit();
	}  else  {
		Log_Trace("client %d received AllocNamedColor reply\n", CLIENT);
		reply_pixel = anreply->pixel;
		Free_Reply(anreply);
	}

	(void) Expect_Nothing(CLIENT);
	Free_Req(req);

	/* store a color in the pixel from the reply */

	Set_Test_Type(CLIENT, test_type);
	req = (xResourceReq *) Make_Req(CLIENT, X_StoreNamedColor);
	Send_Req(CLIENT, (xReq *) req);
	Set_Test_Type(CLIENT, GOOD);
	switch(test_type) {
	case GOOD:
		/* 
		 * The default test code called Expect_Nothing() - this is 
		 * not the case here, we expect BadAccess error.
		 */
		Log_Trace("client %d sent default StoreNamedColor request\n", CLIENT);
		(void) Expect_BadAccess(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	case BAD_LENGTH:
		Log_Trace("client %d sent StoreNamedColor request with bad length (%d)\n", CLIENT, req->length);
		(void) Expect_BadLength(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	case TOO_LONG:
	case JUST_TOO_LONG:
		Log_Trace("client %d sent overlong StoreNamedColor request (%d)\n", CLIENT, req->length);
		(void) Expect_BadLength(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	default:
		Log_Err("INTERNAL ERROR: test_type %d not one of GOOD(%d), BAD_LENGTH(%d), TOO_LONG(%d) or JUST_TOO_LONG(%d)\n",
			test_type, GOOD, BAD_LENGTH, TOO_LONG, JUST_TOO_LONG);
		Abort();
		/*NOTREACHED*/
		break;
	}
	Free_Req(req);
	Exit_OK();
}

int 	tet_thistest;

static void t001(){

int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion StoreNamedColor-1.(C)");
	report_assertion("If the default visual class for screen zero is DirectColor,");
	report_assertion("PseudoColor, or GrayScale: When a client sends a valid");
	report_assertion("StoreNamedColor protocol request to the X server, and the");
	report_assertion("colour cell is read-write, then the X server does not send");
	report_assertion("back an error, event or reply to the client.");

	report_strategy("Call library function testfunc() to do the following:");
	report_strategy("DirectColor, PseudoColor, or GrayScale :");
	report_strategy("Open a connection to the X server using native byte sex.");
	report_strategy("Send a valid StoreNamedColor protocol request to the X server.");
	report_strategy("Verify that the X server does not send back an error, event or reply.");
	report_strategy("Open a connection to the X server using reversed byte sex.");
	report_strategy("Send a valid StoreNamedColor protocol request to the X server.");
	report_strategy("Verify that the X server does not send back an error, event or reply.");

	tpstartup();
	test_type = GOOD;

	/* Call a library function to exercise the test code */
	testfunc(tester_rw);
	tpcleanup();
	pfcount(pass, fail);
}

static void t002(){

int 	pass = 0, fail = 0;

 	report_purpose(2);

	report_assertion("Assertion StoreNamedColor-2.(A)");
	report_assertion("When a client sends a valid StoreNamedColor protocol");
	report_assertion("request to the X server, and the colour cell is read-only,");
	report_assertion("then the X server sends back a BadAccess error to the");
	report_assertion("client.");

	report_strategy("Call library function testfunc() to do the following:");
	report_strategy("Open a connection to the X server using native byte sex.");
	report_strategy("Send a valid StoreNamedColor protocol request to the X server.");
	report_strategy("Verify that the X server sends back a BadAccess error.");
	report_strategy("Open a connection to the X server using reversed byte sex.");
	report_strategy("Send a valid StoreNamedColor protocol request to the X server.");
	report_strategy("Verify that the X server sends back a BadAccess error.");

	tpstartup();
	test_type = GOOD;

	/* Call a library function to exercise the test code */
	testfunc(tester);

	tpcleanup();
	pfcount(pass, fail);
}

static void t003(){

int 	pass = 0, fail = 0;

 	report_purpose(3);

	report_assertion("Assertion StoreNamedColor-3.(A)");
	report_assertion("When a client sends an invalid StoreNamedColor protocol");
	report_assertion("request to the X server, in which the length field of the");
	report_assertion("request is not the minimum length required to contain the");
	report_assertion("request, then the X server sends back a BadLength error to");
	report_assertion("the client.");

	report_strategy("Call library function testfunc() to do the following:");
	report_strategy("Open a connection to the X server using native byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one less than the minimum length required to contain the request.");
	report_strategy("Verify that the X server sends back a BadLength error.");
	report_strategy("Open a connection to the X server using reversed byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one less than the minimum length required to contain the request.");
	report_strategy("Verify that the X server sends back a BadLength error.");
	report_strategy("");
	report_strategy("Open a connection to the X server using native byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one greater than the minimum length required to contain the request.");
	report_strategy("Verify that the X server sends back a BadLength error.");
	report_strategy("Open a connection to the X server using reversed byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one greater than the minimum length required to contain the request.");
	report_strategy("Verify that the X server sends back a BadLength error.");

	tpstartup();
	test_type = BAD_LENGTH; /* < minimum */

	/* Call a library function to exercise the test code */
	testfunc(tester);

	test_type = JUST_TOO_LONG; /* > minimum */

	/* Call a library function to exercise the test code */
	testfunc(tester);

	tpcleanup();
	pfcount(pass, fail);
}

static void t004(){

int 	pass = 0, fail = 0;

 	report_purpose(4);

	report_assertion("Assertion StoreNamedColor-4.(B)");
	report_assertion("When a client sends an invalid StoreNamedColor protocol");
	report_assertion("request to the X server, in which the length field of the");
	report_assertion("request exceeds the maximum length accepted by the X server,");
	report_assertion("then the X server sends back a BadLength error to the");
	report_assertion("client.");
	report_assertion("Reason for omission: There is no known portable test method for this assertion");

	report_strategy("Call library function testfunc() to do the following:");
	report_strategy("Open a connection to the X server using native byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one greater than the maximum length accepted by the server.");
	report_strategy("Verify that the X server sends back a BadLength error.");
	report_strategy("Open a connection to the X server using reversed byte sex.");
	report_strategy("Send an invalid StoreNamedColor protocol request to the X server with length ");
	report_strategy("  one greater than the maximum length accepted by the server.");
	report_strategy("Verify that the X server sends back a BadLength error.");

	tpstartup();
	test_type = TOO_LONG;

	/* Call a library function to exercise the test code */
	testfunc(tester);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = protostartup;
void	(*tet_cleanup)() = protocleanup;
